
/* ===== pfod Command for LinkIt Led Control ====
{.<bg n><w><+3>Control LinkIt Led via Wifi|A~<+6>Led is `0~~Off\On}
 */
// Using Seeed Studio LinkIt ONE board connecting via Wifi
// NOTE: This is experimental. LinkIt ONE does not release the connection if pfodApp does shut down cleanly
// LinkIt ONE then stops accepting connections and needs to be reset

// You need to modify the WIFI_AP, WIFI_PASSWORD and WIFI_AUTH below
// to match your network settings. Set to portNo to what you want. 

// One LinkIt ONE connects to your router, logon to your router and
// lock the IP address for LinkIt ONE in your DHCP settings 

/* Code generated by pfodDesigner V1.2.573
 * (c)2014-2015 Forward Computing and Control Pty. Ltd.
 * NSW Australia, www.forward.com.au
 * This generated code may be freely used for both private and commerical use
 */
#include <LWiFi.h>
#include <LWiFiServer.h>
#include <LWiFiClient.h>

// Download library from http://www.forward.com.au/pfod/pfodParserLibraries/index.html
#include <LEEPROM.h>
#include <pfodLinkIt.h>
pfodLinkIt parser; // create a parser to handle the pfod messages
#define WIFI_AP "nameOfYourWifiAccessPointBroadcasts"
#define WIFI_PASSWORD ""
#define WIFI_AUTH LWIFI_OPEN  // choose from LWIFI_OPEN, LWIFI_WPA, or LWIFI_WEP according to your WiFi AP configuration
const int portNo = 4989;

LWiFiServer server(portNo);
LWiFiClient client;

boolean alreadyConnected = false; // whether or not the client was connected previously

// give the board pins names, if you change the pin number here you will change the pin controlled
int cmd_A_pin = 13; // name the output pin for 'Led is '

// the setup routine runs once on reset:
void setup() {
  for (int i=3; i>0; i--) {
    // wait a few secs to see if we are being programmed
    delay(1000);
  }
  //pinMode(cmd_A_pin, INPUT_PULLUP); 
  pinMode(cmd_A_pin, OUTPUT); // output for 'Led is ' is initially LOW, uncomment line above if you want it initially HIGH
 
  // initialize the LWifi
  LWiFi.begin();
  while (0 == LWiFi.connect(WIFI_AP, LWiFiLoginInfo(WIFI_AUTH, WIFI_PASSWORD))) {
    delay(1000);
  }
  // start listening for clients
  server.begin();
  // initialize client
  client = server.available(); // evaluates to false if no connection

  // <<<<<<<<< Your extra setup code goes here
}

// the loop routine runs over and over again forever:
void loop() {
  if (!client) { // see if a client is available
    client = server.available(); // evaluates to false if no connection
  } else {
    // have client
    if (!client.connected()) {
      if (alreadyConnected) {
      // client closed so clean up
      closeConnection(parser.getPfodAppStream());
      }
    } else {
      // have connected client
      if (!alreadyConnected) {
        parser.connect(&client); // sets new io stream to read from and write to
        alreadyConnected = true;
      }

      byte cmd = parser.parse(); // pass it to the parser
      // parser returns non-zero when a pfod command is fully parsed
      if (cmd != 0) { // have parsed a complete msg { to }
        byte* pfodFirstArg = parser.getFirstArg(); // may point to \0 if no arguments in this msg.
        long pfodLongRtn; // used for parsing long return arguments, if any
        if ('.' == cmd) {
          // pfodApp has connected and sent {.} , it is asking for the main menu
          // send back the menu designed
          sendMainMenu();

        // now handle commands returned from button/sliders
        } else if('A'==cmd) { // user moved slider -- 'Led is '
          // set output based on slider 0 == LOW, 1 == HIGH
          parser.parseLong(pfodFirstArg,&pfodLongRtn); // parse first arg as a long
          digitalWrite(cmd_A_pin,pfodLongRtn); // set output
          sendMainMenuUpdate(); // always send back a pfod msg otherwise pfodApp will disconnect.

        } else if ('!' == cmd) {
          // CloseConnection command
          closeConnection(parser.getPfodAppStream());
        } else {
          // unknown command
          parser.print(F("{}")); // always send back a pfod msg otherwise pfodApp will disconnect.
        }
      }
    }
  }
  
  //  <<<<<<<<<<<  Your other loop() code goes here 
  
}

void closeConnection(Stream *io) {
  // add any special code here to force connection to be dropped
  parser.closeConnection(); // nulls io stream
  alreadyConnected = false;
  if (!client) {
    return;
  } // else
  client.stop();
  client = server.available(); // evaluates to false if no connection
}

void sendMainMenu() {
  parser.print(F("{."));  // start a Menu screen pfod message
  send_menuContents(); // send the menu contents
  parser.print(F("}"));  // close pfod message
}

void sendMainMenuUpdate() {
  parser.print(F("{:"));  // start an Update Menu pfod message
  send_menuContents(); // send the menu contents
  parser.print(F("}"));  // close pfod message
}

// modify this method if you need to update the menu to reflect state changes
void send_menuContents() {
  // send menu prompt
  parser.print(F("<bg n><w><+3>Control LinkIt Led via Wifi"));
  // send menu items
  parser.print(F("|A~<+6>Led is `"));
  parser.print(digitalRead(cmd_A_pin)); // read current output state 0 Low or 1 High
  parser.print(F("~~Off\\On")); // Note the \\ inside the "'s to send \ 
  // ============ end of menu item ===========
}

// ============= end generated code =========
